﻿using Microsoft.PointOfService;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace TestApplication
{
    public partial class MICPageBuilder : Form
    {
        private Microsoft.PointOfService.PosPrinter _printer;
        PrintRotation printRotation = PrintRotation.Normal;

        public MICPageBuilder(Microsoft.PointOfService.PosPrinter _printer)
        {
            InitializeComponent();
            this._printer = _printer;
            _printer.DirectIOEvent += _printer_DirectIOEvent;
            exitPageMode();
            updatePageModePrintArea();
            symbologyComboBox.Items.Add(BarCodeSymbology.Codabar);
            symbologyComboBox.Items.Add(BarCodeSymbology.Code128);
            symbologyComboBox.Items.Add(BarCodeSymbology.Code128Parsed);
            symbologyComboBox.Items.Add(BarCodeSymbology.Code39);
            symbologyComboBox.Items.Add(BarCodeSymbology.DataMatrix);
            symbologyComboBox.Items.Add(BarCodeSymbology.Itf);
            symbologyComboBox.Items.Add(BarCodeSymbology.Pdf417);
            symbologyComboBox.Items.Add(BarCodeSymbology.Other);
        }

        private void rotatePrintButton_Click(object sender, EventArgs e)
        {
            if (rotatePrintBox.Text == "Normal")
                printRotation = PrintRotation.Normal;
            else if (rotatePrintBox.Text == "Left 90")
                printRotation = PrintRotation.Left90;
            else if (rotatePrintBox.Text == "Right 90")
                printRotation = PrintRotation.Right90;
            else
                printRotation = PrintRotation.Rotate180;

            rotatePrintLabel.Text = "Current Rotation: " + rotatePrintBox.Text;

            if (barCodeCheckBox.Checked && rotatePrintBox.Text != "Normal" )
            {
                printRotation |= PrintRotation.Barcode;
                rotatePrintLabel.Text += ", Bar Codes Included";
            }

            if (bitmapCheckBox.Checked && rotatePrintBox.Text != "Normal")
            {
                printRotation |= PrintRotation.Bitmap;
                rotatePrintLabel.Text += ", Bitmaps Included";
            }

            _printer.RotatePrint(_printer.PageModeStation, printRotation);
        }

        private void pageModePrintButton_Click(object sender, EventArgs e)
        {
            PageModePrintControl control = PageModePrintControl.PageMode;
            if (pageModeComboBox.Text == "Enter Page Mode")
            {
                control = PageModePrintControl.PageMode;
                enterPageMode();
            }
            else if (pageModeComboBox.Text == "Print Page")
            {
                control = PageModePrintControl.PrintSave;
            }
            else if (pageModeComboBox.Text == "Print Page and Exit")
            {
                control = PageModePrintControl.Normal;
                exitPageMode();
            }
            else if (pageModeComboBox.Text == "Exit")
            {
                control = PageModePrintControl.Cancel;
                exitPageMode();
            }

            _printer.PageModePrint(control);
            updateScriptBox();
        }

        private void setPageModeControlsEnabled(bool enabled)
        {
            mapModeGroupBox.Enabled = enabled;
            rotationGroupBox.Enabled = enabled;
            textGroupBox.Enabled = enabled;
            bitmapGroupBox.Enabled = enabled;
            barCodeGroupBox.Enabled = enabled;
            lineGroupBox.Enabled = enabled;
            printImmediateGroupBox.Enabled = enabled;
            cutPaperGroupBox.Enabled = enabled;

            horizontalPositionUpDown.Enabled = enabled;
            verticalPositionUpDown.Enabled = enabled;
            printAreaXUpDown.Enabled = enabled;
            printAreaYUpDown.Enabled = enabled;
            printAreaWidthUpDown.Enabled = enabled;
            printAreaHeightUpDown.Enabled = enabled;
            clearPrintAreaButton.Enabled = enabled;
            label1.Enabled = enabled;
            label2.Enabled = enabled;
            label3.Enabled = enabled;
            label4.Enabled = enabled;
            label5.Enabled = enabled;
            label6.Enabled = enabled;
            
        }

        private void exitPageMode()
        {
            pageModeComboBox.Items.Clear();
            pageModeComboBox.Items.Add("Enter Page Mode");

            setPageModeControlsEnabled(false);

            pageModeComboBox.SelectedIndex = 0;
        }

        private void enterPageMode()
        {
            pageModeComboBox.Items.Clear();
            pageModeComboBox.Items.Add("Print Page");
            pageModeComboBox.Items.Add("Print Page and Exit");
            pageModeComboBox.Items.Add("Exit");

            setPageModeControlsEnabled(true);

            pageModeComboBox.SelectedIndex = 0;
        }

        private void printNormalButton_Click(object sender, EventArgs e)
        {
            _printer.PrintNormal(PrinterStation.Slip, normalDataTextBox.Text);
            updateScriptBox();
        }

        private void printBitmapButton_Click(object sender, EventArgs e)
        {
            int alignment;
            if (bitmapAlignmentComboBox.Text == "Left")
                alignment = PosPrinter.PrinterBitmapLeft;
            else if (bitmapAlignmentComboBox.Text == "Center")
                alignment = PosPrinter.PrinterBitmapCenter;
            else
                alignment = PosPrinter.PrinterBitmapRight;
            int width = (int)bitmapWidthUpDown.Value;
            if (!bitmapUseWidthCheckBox.Checked)
                width = PosPrinter.PrinterBitmapAsIs;
            _printer.PrintBitmap(_printer.PageModeStation, bitmapNameTextBox.Text, width, alignment);
            updateScriptBox();
        }

        private void printBarCodeButton_Click(object sender, EventArgs e)
        {
            int alignment;
            if (barCodeAlignmentComboBox.Text == "Left")
                alignment = PosPrinter.PrinterBarCodeLeft;
            else if (barCodeAlignmentComboBox.Text == "Center")
                alignment = PosPrinter.PrinterBarCodeCenter;
            else
                alignment = PosPrinter.PrinterBarCodeRight;
            _printer.PrintBarCode(_printer.PageModeStation, barCodeDataTextBox.Text, (BarCodeSymbology)symbologyComboBox.Items[symbologyComboBox.SelectedIndex], (int)barCodeHeightUpDown.Value, (int)barCodeWidthUpDown.Value, alignment, BarCodeTextPosition.None);
            updateScriptBox();
        }

        private void drawRuledLineButton_Click(object sender, EventArgs e)
        {
            LineStyle style = LineStyle.ChainLine;
            LineDirection direction = LineDirection.Horizontal;
            if (styleComboBox.SelectedItem.ToString() == "Single Solid Line")
                style = LineStyle.SingleSolidLine;
            else if (styleComboBox.SelectedItem.ToString() == "Double Solid Line")
                style = LineStyle.DoubleSolidLine;
            else if (styleComboBox.SelectedItem.ToString() == "Broken Line")
                style = LineStyle.BrokenLine;
            if (directionComboBox.SelectedItem.ToString() == "Vertical")
                direction = LineDirection.Vertical;
            _printer.DrawRuledLine(PrinterStation.Receipt, positionListTextBox.Text, direction, (int)drawRuledLineWidthUpDown.Value, style, 0);
            updateScriptBox();
        }

        private void useWidthCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            bitmapWidthUpDown.Enabled = bitmapUseWidthCheckBox.Checked;
        }

        private void clearPrintAreaButton_Click(object sender, EventArgs e)
        {
            updatePageModePrintArea();
            _printer.ClearPrintArea();
            updateScriptBox();
        }

        private void printAreaXUpDown_ValueChanged(object sender, EventArgs e)
        {
            updatePageModePrintArea();
        }

        private void printAreaYUpDown_ValueChanged(object sender, EventArgs e)
        {
            updatePageModePrintArea();
        }

        private void printAreaWidthUpDown_ValueChanged(object sender, EventArgs e)
        {
            updatePageModePrintArea();
        }

        private void printAreaHeightUpDown_ValueChanged(object sender, EventArgs e)
        {
            updatePageModePrintArea();
        }

        private void updatePageModePrintArea()
        {
            _printer.PageModePrintArea = new Rectangle((int)printAreaXUpDown.Value, (int)printAreaYUpDown.Value, (int)printAreaWidthUpDown.Value, (int)printAreaHeightUpDown.Value);
            pagePanel.Invalidate();
        }

        private void horizontalPositionUpDown_ValueChanged(object sender, EventArgs e)
        {
            _printer.PageModeHorizontalPosition = (int)horizontalPositionUpDown.Value;
            pagePanel.Invalidate();
        }

        private void verticalPositionUpDown_ValueChanged(object sender, EventArgs e)
        {
            _printer.PageModeVerticalPosition = (int)verticalPositionUpDown.Value;
            pagePanel.Invalidate();
        }

        private void setMapModeButton_Click(object sender, EventArgs e)
        {
            if (mapModeComboBox.Text == "English")
                _printer.MapMode = MapMode.English;
            else if (mapModeComboBox.Text == "Metric")
                _printer.MapMode = MapMode.Metric;
            else if (mapModeComboBox.Text == "Twips")
                _printer.MapMode = MapMode.Twips;
            else if (mapModeComboBox.Text == "Dots")
                _printer.MapMode = MapMode.Dots;
            updatePageModePrintArea();
        }

        private void refreshFontListButton_Click(object sender, EventArgs e)
        {
            doFileRefresh();
        }

        private void doFileRefresh()
        {
            Cursor old = Cursor.Current;
            Cursor.Current = Cursors.WaitCursor;
            DirectIOData diodata = _printer.DirectIO(1, 0, null); //get file lists
            populateFileLists(diodata);
            Cursor.Current = old;
        }

        private void populateFileLists(DirectIOData diodata)
        {
            fontListListBox.Items.Clear();
            for (int i = 0; i < _printer.FontTypefaceList.Length; i++)
                fontListListBox.Items.Add((int)(i + 1) + ": " + _printer.FontTypefaceList[i]);

            bitmapListListBox.Items.Clear();
            List<string> bitmaps = (List<string>)diodata.Object;
            for (int i = 0; i < bitmaps.Count; i++)
            {
                if (bitmaps[i] != "")
                    bitmapListListBox.Items.Add(bitmaps[i]);
            }
        }

        List<float[]> scriptCoords = new List<float[]>();
        private void updateScriptBox()
        {
            DirectIOData diodata = _printer.DirectIO(2, 0, null); //retrieve lds2 script
            string script = diodata.Object.ToString();
            scriptBox.Text = script;
            script = script.ToLower();



            bool commaFound = false;
            script = script.Substring(script.IndexOf("d200") + 4);
            script = script.Substring(script.IndexOf(")") + 1);
            string pageWidthStr = "";
            string pageHeightStr = "";
            for (int i = 0; i < script.Length; i++)
            {
                if ((script[i] >= '0' && script[i] <= '9') || script[i] == '.')
                {
                    if (commaFound)
                        pageHeightStr += script[i];
                    else
                        pageWidthStr += script[i];
                }
                else if (script[i] == ',')
                {
                    if (!commaFound)
                        commaFound = true;
                    else
                        break;
                }
                else if (script[i] == '^')
                    break;
            }
            float pageWidth = float.Parse(pageWidthStr);
            float pageHeight = float.Parse(pageHeightStr);

            int fIndex = script.IndexOf("^f");

            scriptCoords.Clear();
            string x = "";
            string y = "";
            while (fIndex >= 0)
            {
                commaFound = false;
                x = "";
                y = "";
                script = script.Substring(fIndex + 2);
                script = script.Substring(script.IndexOf(")") + 1);
                for (int i = 0; i < script.Length; i++)
                {
                    if ((script[i] >= '0' && script[i] <= '9') || script[i] == '.')
                    {
                        if (commaFound)
                            y += script[i];
                        else
                            x += script[i];
                    }
                    else if (script[i] == ',')
                    {
                        if (!commaFound)
                            commaFound = true;
                        else
                            break;
                    }
                    else if (script[i] == '^')
                        break;
                }
                fIndex = script.IndexOf("^f");
                scriptCoords.Add(new float[] { float.Parse(x) / pageWidth, (pageHeight - float.Parse(y)) / pageHeight });
            }
            pagePanel.Invalidate();
        }

        private void pagePanel_Paint(object sender, PaintEventArgs e)
        {
            if (!_printer.DeviceEnabled || !_printer.Claimed)
                return;
            Graphics g = pagePanel.CreateGraphics();
            g.Clear(pagePanel.BackColor);
            float xScale = (float)(pagePanel.Width - 2) / (float)(_printer.PageModeArea.X);
            float yScale = (float)(pagePanel.Height - 2) / (float)(_printer.PageModeArea.Y);
            float scale = Math.Min(xScale, yScale);
            g.FillRectangle(Brushes.White, 0, 0, _printer.PageModeArea.X * scale, _printer.PageModeArea.Y * scale);
            g.DrawRectangle(new Pen(Color.Red), _printer.PageModePrintArea.X * scale, _printer.PageModePrintArea.Y * scale + 1, _printer.PageModePrintArea.Width * scale, _printer.PageModePrintArea.Height * scale);
            g.DrawLine(new Pen(Color.Green), new Point((int)((_printer.PageModePrintArea.X + _printer.PageModeHorizontalPosition) * scale), 0), new Point((int)((_printer.PageModePrintArea.X + _printer.PageModeHorizontalPosition) * scale), pagePanel.Height));
            g.DrawLine(new Pen(Color.Green), new Point(0, (int)((_printer.PageModePrintArea.Y + _printer.PageModeVerticalPosition) * scale)), new Point(pagePanel.Width, (int)((_printer.PageModePrintArea.Y + _printer.PageModeVerticalPosition) * scale)));
            float[] coords;
            for (int i = 0; i < scriptCoords.Count; i++)
            {
                coords = scriptCoords[i];
                g.DrawLine(new Pen(Color.Black), new Point((int)(coords[0] * _printer.PageModeArea.X * scale - 3), (int)(coords[1] * _printer.PageModeArea.Y * scale - 3)), new Point((int)(coords[0] * _printer.PageModeArea.X * scale + 3), (int)(coords[1] * _printer.PageModeArea.Y * scale + 3)));
                g.DrawLine(new Pen(Color.Black), new Point((int)(coords[0] * _printer.PageModeArea.X * scale - 3), (int)(coords[1] * _printer.PageModeArea.Y * scale + 3)), new Point((int)(coords[0] * _printer.PageModeArea.X * scale + 3), (int)(coords[1] * _printer.PageModeArea.Y * scale - 3)));
            }
        }

        private void directIOButton_Click(object sender, EventArgs e)
        {
            DirectIOForm form = new DirectIOForm(_printer);
            form.ShowDialog();
            if (form.tbResultObject.Text.StartsWith("^A)"))
                scriptBox.Text = form.tbResultObject.Text;
            form.tbResultObject.Text = "";
            updatePageModePrintArea();
        }

        private void bitmapListListBox_DoubleClick(object sender, EventArgs e)
        {
            if (bitmapListListBox.Items.Count > bitmapListListBox.SelectedIndex && bitmapListListBox.SelectedIndex >= 0)
                bitmapNameTextBox.Text = bitmapListListBox.Items[bitmapListListBox.SelectedIndex].ToString();
        }

        private void fontListListBox_DoubleClick(object sender, EventArgs e)
        {
            if (fontListListBox.Items.Count > fontListListBox.SelectedIndex && fontListListBox.SelectedIndex >= 0)
            {
                string fontNumber = fontListListBox.Items[fontListListBox.SelectedIndex].ToString().Substring(0, fontListListBox.Items[fontListListBox.SelectedIndex].ToString().IndexOf(':'));
                normalDataTextBox.Text = (char)27 + "|" + fontNumber + "fT";
            }
        }

        private void printImmediateButton_Click(object sender, EventArgs e)
        {
            _printer.PrintImmediate(_printer.PageModeStation, printImmediateTextBox.Text);
        }

        private void setBitmapButton_Click(object sender, EventArgs e)
        {
            int alignment;
            if (bitmapAlignmentComboBox.Text == "Left")
                alignment = PosPrinter.PrinterBitmapLeft;
            else if (bitmapAlignmentComboBox.Text == "Center")
                alignment = PosPrinter.PrinterBitmapCenter;
            else
                alignment = PosPrinter.PrinterBitmapRight;
            int width = (int)bitmapWidthUpDown.Value;
            if (!bitmapUseWidthCheckBox.Checked)
                width = PosPrinter.PrinterBitmapAsIs;
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Monochrome Bitmaps|*.bmp;*.zip";
            if (dlg.ShowDialog() == DialogResult.OK)
                _printer.SetBitmap(0, _printer.PageModeStation, dlg.FileName, width, alignment);
        }

        private void printMemoryBitmapButton_Click(object sender, EventArgs e)
        {
            int alignment;
            if (bitmapAlignmentComboBox.Text == "Left")
                alignment = PosPrinter.PrinterBitmapLeft;
            else if (bitmapAlignmentComboBox.Text == "Center")
                alignment = PosPrinter.PrinterBitmapCenter;
            else
                alignment = PosPrinter.PrinterBitmapRight;
            int width = (int)bitmapWidthUpDown.Value;
            if (!bitmapUseWidthCheckBox.Checked)
                width = PosPrinter.PrinterBitmapAsIs;
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Monochrome Bitmaps|*.bmp;*.zip";
            if (dlg.ShowDialog() == DialogResult.OK)
                _printer.PrintMemoryBitmap(_printer.PageModeStation, new Bitmap(dlg.FileName), width, alignment);
            updateScriptBox();
        }

        private void clearOutputButton_Click(object sender, EventArgs e)
        {
            rotatePrintBox.Text = "Normal";
            rotatePrintButton.PerformClick();
            _printer.ClearOutput();
            exitPageMode();
            updateScriptBox();
        }

        private void pagePanel_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            float xScale = (float)(pagePanel.Width - 2) / (float)(_printer.PageModeArea.X);
            float yScale = (float)(pagePanel.Height - 2) / (float)(_printer.PageModeArea.Y);
            float scale = Math.Min(xScale, yScale);
            if (e.X >= scale * (_printer.PageModeArea.X - 1) || e.Y >= scale * (_printer.PageModeArea.Y - 1))
                return;
            printAreaXUpDown.Value = 0;
            printAreaYUpDown.Value = 0;
            printAreaWidthUpDown.Value = _printer.PageModeArea.X - 1;
            printAreaHeightUpDown.Value = _printer.PageModeArea.Y - 1;
            horizontalPositionUpDown.Value = (int)(e.X / scale);
            verticalPositionUpDown.Value = (int)(e.Y / scale);
            pagePanel.Invalidate();
        }

        private void cutPaperButton_Click(object sender, EventArgs e)
        {
            _printer.CutPaper((int)cutPercentageUpDown.Value);
            updateScriptBox();
            pagePanel.Invalidate();
            exitPageMode();
        }

        private void setFont(string fontName)
        {
            for (int i = 0; i < _printer.FontTypefaceList.Length; i++)
            {
                if (_printer.FontTypefaceList[i] == fontName)
                {
                    string fontNumber = (i + 1).ToString();
                    _printer.PrintNormal(PrinterStation.Receipt, (char)27 + "|" + fontNumber + "fT");
                    break;
                }
            }
        }

        enum DataMatrixEncodingScheme { auto = 0, ASCII = 1, C40 = 2, Text = 3, Base256 = 4 }
        private BarCodeSymbology generateDataMatrixSymbology(int leftDimension, int rightDimension, bool tildeIsControlCharacter, DataMatrixEncodingScheme scheme)
        {
            BarCodeSymbology symbology = BarCodeSymbology.DataMatrix;
            symbology |= (BarCodeSymbology)(leftDimension << 24);
            symbology |= (BarCodeSymbology)(rightDimension << 16);
            symbology |= (BarCodeSymbology)(((int)scheme) << 13);
            if (tildeIsControlCharacter)
                symbology |= (BarCodeSymbology)(1 << 12);
            return symbology;
        }

        private BarCodeSymbology generatePDF417Symbology(int scaleVertically, int scaleHorizontally, int codewords, int ECCNumber)
        {
            BarCodeSymbology symbology = BarCodeSymbology.Pdf417;
            symbology |= (BarCodeSymbology)(scaleVertically << 25);
            symbology |= (BarCodeSymbology)(scaleHorizontally << 18);
            symbology |= (BarCodeSymbology)(codewords << 13);
            symbology |= (BarCodeSymbology)(ECCNumber << 9);
            return symbology;
        }

        void _printer_DirectIOEvent(object sender, DirectIOEventArgs e)
        {
            Console.WriteLine("caught DirectIOEvent: " + e.Object.ToString());
        }

        private void checkHealthButton_Click(object sender, EventArgs e)
        {
            checkHealthTextBox.Text = "";
            string ch = _printer.CheckHealth((HealthCheckLevel)(((int)HealthCheckLevel.Interactive) | 0x40 | 0x20 | 0x10 | 0x8 | 0x4));
            checkHealthTextBox.Text = ch;
        }
    }
}